declare class BebopRuntimeError extends Error {
    constructor(message: string);
}
/**
 * Represents a globally unique identifier (GUID).
 */
declare class Guid {
    private readonly value;
    static readonly empty: Guid;
    /**
       * Constructs a new Guid object with the specified value.
       * @param value The value of the GUID.
       */
    private constructor();
    /**
     * Gets the string value of the Guid.
     * @returns The string representation of the Guid.
     */
    toString(): string;
    /**
      * Checks if the Guid is empty.
      * @returns true if the Guid is empty, false otherwise.
      */
    isEmpty(): boolean;
    /**
     * Checks if a value is a Guid.
     * @param value The value to be checked.
     * @returns true if the value is a Guid, false otherwise.
     */
    static isGuid(value: any): value is Guid;
    /**
    * Parses a string into a Guid.
    * @param value The string to be parsed.
    * @returns A new Guid that represents the parsed value.
    * @throws {BebopRuntimeError} If the input string is not a valid Guid.
    */
    static parseGuid(value: string): Guid;
    /**
    * Creates a new Guid.
    * @returns A new Guid.
    */
    static newGuid(): Guid;
    /**
    * Checks if the Guid is equal to another Guid.
    * @param other The other Guid to be compared with.
    * @returns true if the Guids are equal, false otherwise.
    */
    equals(other: Guid): boolean;
    /**
    * Writes the Guid to a DataView.
    * @param view The DataView to write to.
    * @param length The position to start writing at.
    */
    writeToView(view: DataView, length: number): void;
    /**
    * Creates a Guid from a byte array.
    * @param buffer The byte array to create the Guid from.
    * @param index The position in the array to start reading from.
    * @returns A new Guid that represents the byte array.
    */
    static fromBytes(buffer: Uint8Array, index: number): Guid;
}
/**
 * Represents a wrapper around the `Map` class with support for using `Guid` instances as keys.
 *
 * This class is designed to provide a 1:1 mapping between `Guid` instances and values, allowing `Guid` instances to be used as keys in the map.
 * The class handles converting `Guid` instances to their string representation for key storage and retrieval.
 * @remarks this is required because Javascript lacks true reference equality. Thus two `Guid` instances with the same value are not equal.
 */
declare class GuidMap<TValue> {
    private readonly map;
    /**
     * Creates a new GuidMap instance.
     * @param entries - An optional array or iterable containing key-value pairs to initialize the map.
     */
    constructor(entries?: readonly (readonly [Guid, TValue])[] | null | Iterable<readonly [Guid, TValue]>);
    /**
     * Sets the value associated with the specified `Guid` key in the map.
     * @param key The `Guid` key.
     * @param value The value to be set.
     * @returns The updated `GuidMap` instance.
     */
    set(key: Guid, value: TValue): this;
    /**
     * Retrieves the value associated with the specified `Guid` key from the map.
     * @param key The `Guid` key.
     * @returns The associated value, or `undefined` if the key is not found.
     */
    get(key: Guid): TValue | undefined;
    /**
     * Deletes the value associated with the specified `Guid` key from the map.
     * @param key The `Guid` key.
     * @returns `true` if the key was found and deleted, or `false` otherwise.
     */
    delete(key: Guid): boolean;
    /**
     * Checks if the map contains the specified `Guid` key.
     * @param key The `Guid` key.
     * @returns `true` if the key is found, or `false` otherwise.
     */
    has(key: Guid): boolean;
    /**
     * Removes all entries from the map.
     */
    clear(): void;
    /**
     * Returns the number of entries in the map.
     * @returns The number of entries in the map.
     */
    get size(): number;
    /**
     * Executes the provided callback function once for each key-value pair in the map.
     * @param callbackFn The callback function to execute.
     */
    forEach(callbackFn: (value: TValue, key: Guid, map: GuidMap<TValue>) => void): void;
    /**
     * Returns an iterator that yields key-value pairs in the map.
     * @returns An iterator for key-value pairs in the map.
     */
    entries(): Generator<[Guid, TValue]>;
    /**
     * Returns an iterator that yields the keys of the map.
     * @returns An iterator for the keys of the map.
     */
    keys(): Generator<Guid>;
    /**
     * Returns an iterator that yields the values in the map.
     * @returns An iterator for the values in the map.
     */
    values(): Generator<TValue>;
    /**
     * Returns an iterator that yields key-value pairs in the map.
     * This method is invoked when using the spread operator or destructuring the map.
     * @returns An iterator for key-value pairs in the map.
     */
    [Symbol.iterator](): Generator<[Guid, TValue]>;
    /**
     * The constructor function used to create derived objects.
     */
    get [Symbol.species](): typeof GuidMap;
}
/**
 * An interface which all generated Bebop interfaces implement.
 * @note this interface is not currently used by the runtime; it is reserved for future use.
 */
interface BebopRecord {
}
declare class BebopView {
    private static textDecoder;
    private static writeBuffer;
    private static writeBufferView;
    private static instance;
    static getInstance(): BebopView;
    minimumTextDecoderLength: number;
    private buffer;
    private view;
    index: number;
    length: number;
    private constructor();
    startReading(buffer: Uint8Array): void;
    startWriting(): void;
    private guaranteeBufferLength;
    private growBy;
    skip(amount: number): void;
    toArray(): Uint8Array;
    readByte(): number;
    readUint16(): number;
    readInt16(): number;
    readUint32(): number;
    readInt32(): number;
    readUint64(): bigint;
    readInt64(): bigint;
    readFloat32(): number;
    readFloat64(): number;
    writeByte(value: number): void;
    writeUint16(value: number): void;
    writeInt16(value: number): void;
    writeUint32(value: number): void;
    writeInt32(value: number): void;
    writeUint64(value: bigint): void;
    writeInt64(value: bigint): void;
    writeFloat32(value: number): void;
    writeFloat64(value: number): void;
    readBytes(): Uint8Array;
    writeBytes(value: Uint8Array): void;
    /**
     * Reads a length-prefixed UTF-8-encoded string.
     */
    readString(): string;
    /**
     * Writes a length-prefixed UTF-8-encoded string.
     */
    writeString(value: string): void;
    readGuid(): Guid;
    writeGuid(value: Guid): void;
    readDate(): Date;
    writeDate(date: Date): void;
    /**
     * Reserve some space to write a message's length prefix, and return its index.
     * The length is stored as a little-endian fixed-width unsigned 32-bit integer, so 4 bytes are reserved.
     */
    reserveMessageLength(): number;
    /**
     * Fill in a message's length prefix.
     */
    fillMessageLength(position: number, messageLength: number): void;
    /**
     * Read out a message's length prefix.
     */
    readMessageLength(): number;
}
/**
 * A collection of functions for working with Bebop-encoded JSON.
 */
declare const BebopJson: {
    /**
     * A custom replacer function for JSON.stringify that supports BigInt, Map,
     * Date, Uint8Array, including BigInt values inside Map and Array.
     * @param _key - The key of the property being stringified.
     * @param value - The value of the property being stringified.
     * @returns The modified value for the property, or the original value if not a BigInt or Map.
     */
    replacer: (_key: string | number, value: any) => any;
    /**
     * A custom reviver function for JSON.parse that supports BigInt, Map, Date,
     * Uint8Array, including nested values
     * @param _key - The key of the property being parsed.
     * @param value - The value of the property being parsed.
     * @returns The modified value for the property, or the original value if not a marked type.
     */
    reviver: (_key: string | number, value: any) => any;
};
/**
 * This object contains functions for ensuring that values conform to specific types.
 */
declare const BebopTypeGuard: {
    /**
     * Ensures that the given value is a valid boolean.
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid boolean.
     */
    ensureBoolean: (value: any) => void;
    /**
     * Ensures that the given value is a valid Uint8 number (0 to 255).
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid Uint8 number.
     */
    ensureUint8: (value: any) => void;
    /**
     * Ensures that the given value is a valid Int16 number (-32768 to 32767).
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid Int16 number.
     */
    ensureInt16: (value: any) => void;
    /**
     * Ensures that the given value is a valid Uint16 number (0 to 65535).
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid Uint16 number.
     */
    ensureUint16: (value: any) => void;
    /**
     * Ensures that the given value is a valid Int32 number (-2147483648 to 2147483647).
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid Int32 number.
     */
    ensureInt32: (value: any) => void;
    /**
     * Ensures that the given value is a valid Uint32 number (0 to 4294967295).
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid Uint32 number.
     */
    ensureUint32: (value: any) => void;
    /**
     * Ensures that the given value is a valid Int64 number (-9223372036854775808 to 9223372036854775807).
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid Int64 number.
     */
    ensureInt64: (value: bigint | number) => void;
    /**
     * Ensures that the given value is a valid Uint64 number (0 to 18446744073709551615).
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid Uint64 number.
     */
    ensureUint64: (value: bigint | number) => void;
    /**
     * Ensures that the given value is a valid BigInt number.
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid BigInt number.
     */
    ensureBigInt: (value: any) => void;
    /**
     * Ensures that the given value is a valid float number.
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid float number.
     */
    ensureFloat: (value: any) => void;
    /**
     * Ensures that the given value is a valid Map object, with keys and values that pass the specified validators.
     * @param value - The value to check.
     * @param keyTypeValidator - A function that validates the type of each key in the Map.
     * @param valueTypeValidator - A function that validates the type of each value in the Map.
     * @throws {BebopRuntimeError} - If the value is not a valid Map object, or if any key or value fails validation.
     */
    ensureMap: (value: any, keyTypeValidator: (key: any) => void, valueTypeValidator: (value: any) => void) => void;
    /**
     * Ensures that the given value is a valid Array object, with elements that pass the specified validator.
     * @param value - The value to check.
     * @param elementTypeValidator - A function that validates the type of each element in the Array.
     * @throws {BebopRuntimeError} - If the value is not a valid Array object, or if any element fails validation.
     */
    ensureArray: (value: any, elementTypeValidator: (element: any) => void) => void;
    /**
     * Ensures that the given value is a valid Date object.
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid Date object.
     */
    ensureDate: (value: any) => void;
    /**
     * Ensures that the given value is a valid Uint8Array object.
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid Uint8Array object.
     */
    ensureUint8Array: (value: any) => void;
    /**
     * Ensures that the given value is a valid string.
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid string.
     */
    ensureString: (value: any) => void;
    /**
     * Ensures that the given value is a valid enum value.
     * @param value - The value to check.
     * @param enumValues - An array of valid enum values.
     * @throws {BebopRuntimeError} - If the value is not a valid enum value.
     */
    ensureEnum: (value: any, enumValue: object) => void;
    /**
     * Ensures that the given value is a valid GUID string.
     * @param value - The value to check.
     * @throws {BebopRuntimeError} - If the value is not a valid GUID string.
     */
    ensureGuid: (value: any) => void;
};

export { BebopJson, BebopRecord, BebopRuntimeError, BebopTypeGuard, BebopView, Guid, GuidMap };
